@@ -5,7 +5,7 @@ use std::str;
 use chrono::{Date, DateTime, Utc, MAX_DATE, MIN_DATE};
 use chrono_english::{parse_date_string, Dialect};
 use color_eyre::Help;
-use dialoguer::{Editor, Input};
+use dialoguer::{Confirm, Editor, Input};
 use syntect::highlighting::Style;
 use syntect::util::as_24_bit_terminal_escaped;
 
@@ -23,33 +23,62 @@ pub const NAME: &str = "the-way";
 /// ASCII code of semicolon
 pub const SEMICOLON: u8 = 59;
 
-#[cfg(target_os = "linux")]
-mod copy {
-    pub const COMMAND: &str = "xclip";
-    pub const ARGS: [&str; 3] = ["-in", "-selection", "clipboard"];
-}
-
-#[cfg(target_os = "macos")]
-mod copy {
-    pub const COMMAND: &str = "pbcopy";
-    pub const ARGS: [&str; 0] = [];
-}
-
-#[cfg(target_os = "android")]
-mod copy {
-    pub const COMMAND: &str = "termux-clipboard-set";
-    pub const ARGS: [&str; 0] = [];
+/// Defines the default supported clipboard copy commands.
+/// A `String` containing the copy command with the arguments is returned
+/// according to the conditional compilation on the detected OS.
+pub(crate) fn get_default_copy_cmd() -> Option<String> {
+    if cfg!(target_os = "linux") {
+        Some("xclip -in -selection clipboard".to_string())
+    } else if cfg!(target_os = "macos") {
+        Some("pbcopy".to_string())
+    } else if cfg!(target_os = "android") {
+        Some("termux-clipboard-set".to_string())
+    } else {
+        None
+    }
 }
 
 /// Set clipboard contents to text
 /// See [issue](https://github.com/aweinstock314/rust-clipboard/issues/28#issuecomment-534295371)
-pub fn copy_to_clipboard(text: &str) -> color_eyre::Result<()> {
-    let mut child = Command::new(copy::COMMAND)
-        .args(&copy::ARGS)
+pub fn copy_to_clipboard(copy_cmd_field: &Option<String>, text: &str) -> color_eyre::Result<()> {
+    let copy_cmd_vec = copy_cmd_field
+        .as_ref()
+        .ok_or(LostTheWay::NoDefaultCopyCommand)?
+        .trim()
+        .split_whitespace()
+        .map(|s| s.to_owned())
+        .collect::<Vec<String>>();
+
+    let default_copy_cmd_vec: Vec<String>;
+    let (copy_cmd, copy_args) = match copy_cmd_vec.split_first() {
+        Some((cmd, args)) => (cmd, args),
+        _ => {
+            default_copy_cmd_vec = get_default_copy_cmd()
+                .ok_or(LostTheWay::NoDefaultCopyCommand)?
+                .split_whitespace()
+                .map(|s| s.to_owned())
+                .collect();
+            let (cmd, args) = match default_copy_cmd_vec.split_first() {
+                Some((cmd, args)) => (cmd, args),
+                // Should never fails due to previous checking
+                _ => unreachable!(),
+            };
+            eprintln!("The `copy_cmd` field is empty, defaulting to `{}`", cmd);
+            (cmd, args)
+        }
+    };
+
+    let mut child = Command::new(copy_cmd)
+        .args(copy_args)
         .stdin(Stdio::piped())
         .spawn()
-        .map_err(|_| LostTheWay::ClipboardError {
-            message: format!("is {} available?", copy::COMMAND),
+        .map_err(|_e| LostTheWay::ClipboardError {
+            message: format!(
+                "is {} available? Also check your `copy_cmd` settings ({})",
+                copy_cmd,
+                // Never fails as it's checked above
+                copy_cmd_field.as_ref().unwrap()
+            ),
         })?;
 
     // When stdin is dropped the fd is automatically closed. See
@@ -61,7 +90,7 @@ pub fn copy_to_clipboard(text: &str) -> color_eyre::Result<()> {
         stdin.write_all(text.as_bytes())?;
     }
 
-    // Wait on pbcopy/xclip to finish.
+    // Wait on copy command to finish.
     child.wait()?;
 
     Ok(())
@@ -71,7 +100,7 @@ pub fn copy_to_clipboard(text: &str) -> color_eyre::Result<()> {
 pub fn split_tags(input: &str) -> Vec<String> {
     input
         .split(' ')
-        .map(|word| word.trim().to_string())
+        .map(|word| word.trim().to_owned())
         .collect::<Vec<String>>()
 }
 
@@ -93,7 +122,7 @@ pub fn split_indices_usize(index_list: &[u8]) -> color_eyre::Result<Vec<usize>>
 pub fn make_indices_string(index_list: &[usize]) -> color_eyre::Result<Vec<u8>> {
     Ok(index_list
         .iter()
-        .map(|index| index.to_string())
+        .map(std::string::ToString::to_string)
         .collect::<Vec<String>>()
         .join(str::from_utf8(&[SEMICOLON])?)
         .as_bytes()
@@ -167,6 +196,16 @@ pub fn user_input(
     }
 }
 
+/// Get a yes/no answer from the user
+pub fn confirm(prompt: &str, default: bool) -> color_eyre::Result<bool> {
+    let theme = dialoguer::theme::ColorfulTheme::default();
+    Ok(Confirm::with_theme(&theme)
+        .with_prompt(prompt)
+        .default(default)
+        .show_default(false)
+        .interact()?)
+}
+
 /// Make an indicatif spinner with given message
 pub fn get_spinner(message: &str) -> indicatif::ProgressBar {
     let spinner = indicatif::ProgressBar::new_spinner();
@@ -180,3 +219,41 @@ pub fn highlight_string(line: &str, style: Style) -> String {
     s.push_str(END_ANSI);
     s
 }
+
+/// Color strings for the terminal
+pub fn highlight_strings(inputs: &[(Style, String)], bg: bool) -> String {
+    if bg {
+        let mut s = String::new();
+        for (style, line) in inputs {
+            s.push_str(&as_24_bit_terminal_escaped(&[(*style, line)], true));
+            s.push_str(END_ANSI);
+        }
+        s
+    } else {
+        as_24_bit_terminal_escaped(
+            &inputs
+                .iter()
+                .map(|(style, line)| (*style, line.as_ref()))
+                .collect::<Vec<_>>(),
+            false,
+        )
+    }
+}
+
+/// Print with color if stdout is tty else without
+pub fn smart_print(inputs: &[(Style, String)], bg: bool, colorize: bool) -> color_eyre::Result<()> {
+    write!(
+        grep_cli::stdout(termcolor::ColorChoice::Auto),
+        "{}",
+        if grep_cli::is_tty_stdout() | colorize {
+            highlight_strings(inputs, bg)
+        } else {
+            inputs
+                .iter()
+                .map(|(_, s)| s.to_string())
+                .collect::<Vec<_>>()
+                .join("")
+        }
+    )?;
+    Ok(())
+}
